//*****************************************
// Photo Box with ATmega64 or Atmega2560
// CPU Clock:O16MHz Ext,Crystal 16MHz)
//   iq[Yf[^@WȂŊOUɂj
//
// File Name:port_bit.c
//
//  2012/06/13 ishi New
//  2023/03/07 Modify for mega328_i2c_old Taishi
//  2023/05/13 Modify for mega328_iic_oled_rtc Taishi
//  2023/12/09 Modify for Photo Box with ATmega64 Taishi
//  2024/01/09 Modify for Photo Box with ATmega64 or ATmega2560 Taishi
//*****************************************

#include	<avr/io.h>

#include	"pr_mega64_2560_photo_box_h.h"

#include	"timer_h.h"
#include	"port_bit_h.h"

unsigned char	gReq_red_on = 0;
unsigned char	gReq_red_off = 0;
unsigned char	gReq_blu_on = 0;
unsigned char	gReq_blu_off = 0;

static	unsigned char	sw1_continue_count = 0;
static	unsigned char	sw2_continue_count = 0;

#ifdef	cpu64
//***************************
//  kdc֌W̏ݒ mega64
//***************************
void	led_init(void)
{
	DDRG |= ((1 << DDG0) | (1 << DDG3));	// PORTG D0 & D3 OUTPUT

	PORTG &= ~((1 << PG0) | (1 << PG3));	// RED & BLU LED OFF

}
#else
 #ifdef cpu2560
//***************************
//  kdc֌W̏ݒ mega2560
//***************************
void	led_init(void)
{
	DDRB |= ((1 << DDB6) | (1 << DDB7));	// PORTB D6(RED) & D7(BLUE) OUTPUT
	DDRG |= ((1 << DDG4) | (1 << DDG5));	// PORTG D4 & D5 OUTPUT LED sp1 & sp2

	PORTB &= ~((1 << PB6) | (1 << PB7));	// RED(D6) & BLU(D7) LED OFF
	PORTG &= ~((1 << PG4) | (1 << PG5));	// LED sp1 & sp2 OFF

}
 #endif
#endif

//******************************
// sp1 LED ON/OFF mega2560
//  on_off : 1=ON, 0=OFF
// *****************************
void	led_sp1_on_off(unsigned char on_off)
{
	if(on_off == 0){
		led_sp1_off();		// LED OFF
	}else{
		led_sp1_on();		// LED ON
	}
}

//******************************
// sp2 LED ON/OFF mega2560
//  on_off : 1=ON, 0=OFF
// *****************************
void	led_sp2_on_off(unsigned char on_off)
{
	if(on_off == 0){
		led_sp2_off();		// LED OFF
	}else{
		led_sp2_on();		// LED ON
	}
}

//******************************
// RED LED ON/OFF mega2560
//  on_off : 1=ON, 0=OFF
//  ݉͎̎gȂ
// *****************************
void	led_red_on_off(unsigned char on_off)
{
	if(on_off == 0){
		led_red_off();		// LED OFF
	}else{
		led_red_on();		// LED ON
	}
}

//******************************
// BLUE LED ON/OFF mega2560
//  on_off : 1=ON, 0=OFF
//  ݉͎̎gȂ
// *****************************
void	led_blu_on_off(unsigned char on_off)
{
	if(on_off == 0){
		led_blu_off();		// LED OFF
	}else{
		led_blu_on();		// LED ON
	}
}

//******************************
// RED & BLUE LED ON/OFF
//  Lϐ
//   gReq_red_on,gReq_red_off,gReq_blu_on,gReq_blu_off
//   lĎۂ̃|[gɏo͂B
//   ^C}[荞݂̒Ă΂̂
//	 Timer2 ̊荞݂gpĂԂKv
// *****************************
void	led_on_off(void)
{
	if(gReq_red_on != 0){
		gReq_red_on = 0;
		led_red_on();
	}

	if(gReq_red_off != 0){
		gReq_red_off = 0;
		led_red_off();
	}

	if(gReq_blu_on != 0){
		gReq_blu_on = 0;
		led_blu_on();
	}

	if(gReq_blu_off != 0){
		gReq_blu_off = 0;
		led_blu_off();
	}
}


static	unsigned char	sw[4];							// SW[1] Ɍ݂̏ԁAĂ镨'1'̏ԁA SW[2] ɉꂽuԂ'1'̏ԁA SW[3] ɗꂽuԂ'1'̏
static	unsigned char	old_sw;
//***************************
//@XCb`Ԃ
// SW[1] Ɍ݂̏ԁAĂ镨'1'̏
// SW[2] ɉꂽuԂ'1'̏
//  Timer2 ̊荞݂gpĂԂKv
//***************************
void	get_sw(void)
{
	unsigned char	uc;

	if(tim_10ms_sw == 0){
		tim_10ms_sw = 3;									// ROrԊu
		uc = ((~SW_IN_PORT) & (SW1_ON_BIT | SW2_ON_BIT));				// ]ăXCb`f[^擾
		if (old_sw == uc){									// OƓf[^̎
			sw[0] = sw[1];
			sw[1] = uc;
			sw[2] = ((sw[0] ^ sw[1]) & sw[1]);				// ꂽXCb`f[^
			sw[3] = ((sw[0] ^ sw[1]) & ~sw[1]);				// ꂽXCb`f[^
		}else{
			sw[2] = 0;
			sw[3] = 0;
		}
		old_sw = uc;										// Of[^ɓ
	}else{
		sw[2] = 0;
		sw[3] = 0;
	}
}

#ifdef	cpu64
//***************************
//  XCb`֌W̏ݒ mega64
//***************************
void	sw_init(void)
{
	DDRG &= ~((1 << DDG1));		// Port-G Bit1 : Intput
	DDRG &= ~((1 << DDG2));		// Port-G Bit2 : Intput
	PORTG |= ((1 << PG1) );		// Port-G Bit1 : Pullup
	PORTG |= ((1 << PG2) );		// Port-G Bit2 : Pullup
	sw1_continue_count = 0;
	sw2_continue_count = 0;
}
#else
 #ifdef cpu2560
//***************************
//  XCb`֌W̏ݒ mega2560
//***************************
void	sw_init(void)
{
	DDRB &= ~((1 << DDB4));		// Port-B Bit4 : Intput
	DDRB &= ~((1 << DDB5));		// Port-B Bit5 : Intput
	PORTB |= ((1 << PB4) );		// Port-B Bit4 : Pullup
	PORTB |= ((1 << PB5) );		// Port-B Bit5 : Pullup
	sw1_continue_count = 0;
	sw2_continue_count = 0;
}
 #endif
#endif


#define			TMR_LONG			100			// 10mSx100=1Sec ̎Ԗ͒ZԂƂ
#define			TMR_CONTIMUE		100			// 10mSx100=1Sec ꑱ@̊ԊuŁuSWR_ON_CONTINUEvɂȂ
#define			TMR_CONTIMUE2		50			// 10mSx50=500mS ꑱ̂Qxڈȍ~́uSWR_ON_CONTINUEvo^C~OɂȂ
//***************************
//@XCb`Ԃ
// SW[1] Ɍ݂̏ԁAĂ镨'1'̏
// SW[2] ɉꂽuԂ'1'̏
//  Timer2 ̊荞݂gpĂԂKv
//***************************
unsigned short	get_action_sw(void)
{
	unsigned short	us=0;

	get_sw();												// XCb`Ԃ

	// SW-1(SET)
	if((sw[2] & SW1_ON_BIT) != 0){							// XCb`Pnm̏uԂ̎(set switch)
		us |= (SW1_ON_NOW | SW1_ON);
		tim_10ms_sw1_continue = TMR_CONTIMUE;
		tim_10ms_sw1_length = TMR_LONG;
		sw1_continue_count = 1;
	}else if((sw[1] & SW1_ON_BIT) != 0){					// XCb`Pnm̎(set switch)
		us |= SW1_ON;
		if(tim_10ms_sw1_continue == 0){						// ꑱo^C~O̎
			us |= SW1_ON_CONTINUE;
			if(sw1_continue_count < 2){
				sw1_continue_count++;
				tim_10ms_sw1_continue = TMR_CONTIMUE;
			}else{
				tim_10ms_sw1_continue = TMR_CONTIMUE2;
			}
		}
	}else if((sw[3] & SW1_ON_BIT) != 0){					// XCb`PꂽuԂ̎(set switch)
		if(tim_10ms_sw1_length == 0){						// ꂽԂ
			us |= SW1_OFF_LONG;
		}else{												// ꂽԂZ
			us |= SW1_OFF_SHORT;
		}
		tim_10ms_sw1_continue = 0;
		tim_10ms_sw1_length = 0;
		sw1_continue_count = 0;
	}else if((sw[1] & SW1_ON_BIT) == 0){					// XCb`Pnee̎(set switch)
		tim_10ms_sw1_continue = 0;
		tim_10ms_sw1_length = 0;
		sw1_continue_count = 0;
	}

	// SW-2(SELECT)
	if((sw[2] & SW2_ON_BIT) != 0){							// XCb`Qnm̏uԂ̎(select switch)
		us |= (SW2_ON_NOW | SW2_ON);
		tim_10ms_sw2_continue = TMR_CONTIMUE;
		tim_10ms_sw2_length = TMR_LONG;
		sw2_continue_count = 1;
	}else if((sw[1] & SW2_ON_BIT) != 0){					// XCb`Qnm̎(select switch)
		us |= SW2_ON;
		if(tim_10ms_sw2_continue == 0){						// ꑱo^C~O̎
			us |= SW2_ON_CONTINUE;
			if(sw2_continue_count < 2){
				sw2_continue_count++;
				tim_10ms_sw2_continue = TMR_CONTIMUE;
			}else{
				tim_10ms_sw2_continue = TMR_CONTIMUE2;
			}
		}
	}else if((sw[3] & SW2_ON_BIT) != 0){					// XCb`QꂽuԂ̎(select switch)
		if(tim_10ms_sw2_length == 0){						// ꂽԂ
			us |= SW2_OFF_LONG;
		}else{												// ꂽԂZ
			us |= SW2_OFF_SHORT;
		}
		tim_10ms_sw2_continue = 0;
		tim_10ms_sw2_length = 0;
		sw2_continue_count = 0;
	}else if((sw[1] & SW2_ON_BIT) == 0){					// XCb`Qnee̎(select switch)
		tim_10ms_sw2_continue = 0;
		tim_10ms_sw2_length = 0;
		sw2_continue_count = 0;
	}

	return(us);
}

//*****************************************
// Q̃XCb`Q@܂ŉiv[v
//  XCb`͂Ɋ荞݃^C}[gB
//  Timer2 ̊荞݂gpĂԂKv
//*****************************************
void	wait_key_off(void)
{
	while(1){											// ̂rv܂ő҂
		get_action_sw();								//@XCb`Ԃ
		if( (sw[1] & (SW1_ON_BIT | SW2_ON_BIT)) == 0 ){		// ꂽ
			break;
		}
	}
}
//*****************************************
// Q̃XCb`Q@܂ŉiv[v
//  XCb`rbǵA`^Op^C}[Ɂ@荞݂͎gȂ
//  \tgEFA^C}[gp
//*****************************************
void	wait_sw_off(void)
{
	while(1){
		if((SW_IN_PORT & (SW1_ON_BIT | SW2_ON_BIT)) == (SW1_ON_BIT | SW2_ON_BIT)){
			delay_ms(100);												// 100mS Delay
			if((SW_IN_PORT & (SW1_ON_BIT | SW2_ON_BIT)) == (SW1_ON_BIT | SW2_ON_BIT)){
				break;													// {ɗꂽƂ
			}
		}
	}
}
//*****************************************
//  rPXCb`irdkdbsjꂽÃ`FbN
//  XCb`Ă    P߂
//  @@@@@ĂȂ  O߂
//
//  XCb`rbǵA`^Op^C}[Ɂ@荞݂͎gȂ
//  \tgEFA^C}[gp
//*****************************************
unsigned char	check_s1_sw_on(void)
{
	unsigned char	uc = 0;

	if((SW_IN_PORT & SW1_ON_BIT) == 0){		// SW1 ON
		delay_ms(100);						// 100mS Delay
		if((SW_IN_PORT & SW1_ON_BIT) == 0){	// SW1 ON Qx
			uc = 1;
		}
	}

	return(uc);
}
//*****************************************
//  rQXCb`irdsjꂽÃ`FbN
//  XCb`Ă    P߂
//  @@@@@ĂȂ  O߂
//
//  XCb`rbǵA`^Op^C}[Ɂ@荞݂͎gȂ
//  \tgEFA^C}[gp
//*****************************************
unsigned char	check_s2_sw_on(void)
{
	unsigned char	uc = 0;

	if((SW_IN_PORT & SW2_ON_BIT) == 0){		// SW2 ON
		delay_ms(100);						// 100mS Delay
		if((SW_IN_PORT & SW2_ON_BIT) == 0){	// SW2 ON Qx
			uc = 1;
		}
	}

	return(uc);
}
